@@ -3,6 +3,8 @@ import datetime
 import decimal
 import json
 from copy import deepcopy
+from functools import reduce
+
 import pytz
 import redis
 
@@ -292,9 +294,8 @@ class RedisForeignKey(RedisNumber):
         self.deserialize_value_check_null(value, redis_root)
         if value not in ['null', None]:
             check_types(value, int)
-            model_name = self.model.__name__
             instance_id = value
-            instance_qs = redis_root._get_instances_by_key(f'{redis_root.prefix}:{model_name}:{instance_id}')
+            instance_qs = redis_root.get(self.model, return_dict=True, id=instance_id)
             if instance_id in instance_qs.keys():
                 value = instance_qs[instance_id]
             else:
@@ -331,15 +332,9 @@ class RedisManyToMany(RedisList):
         self.deserialize_value_check_null(value, redis_root)
         if value not in ['null', None]:
             value = super().deserialize_value(value, redis_root)
-            model_name = self.model.__name__
-            isntances_ids = value
-            instances_list = []
-            for instance_id in isntances_ids:
-                instances = redis_root._get_instances_by_key(f'{redis_root.prefix}:{model_name}:{instance_id}')
-                if len(instances.keys()):
-                    instance = instances[instance_id]
-                    instances_list.append(instance)
-            value = instances_list
+            instances_ids = value
+            instances = redis_root.get(self.model, id_in=instances_ids)
+            value = instances
         
         return value
 
@@ -437,13 +432,13 @@ class RedisRoot:
         if self.stable_non_blocking:
             return bool(int(self.redis_instance.get(f'__creating__:{self.prefix}')))
         else:
-            return self.creating
+            return self.is_creating
     
     def set_wait_creating(self, is_creating):
         if self.stable_non_blocking:
             self.redis_instance.set(f'__creating__:{self.prefix}', int(is_creating))
         else:
-            self.creating = is_creating
+            self.is_creating = is_creating
     
     def wait_creation(self):
         while self.get_wait_creating():
@@ -485,33 +480,54 @@ class RedisRoot:
     ### GET ###
     
     def get(self, model, return_dict=False, **filters):
-        instances = self._get_all_model_instances(model, filters)
+        instances = self._get_model_instances(model, filters)
         result = self._return_with_format(instances, return_dict)
         return result
     
-    def _get_all_model_instances(self, model, filters=None):
-        if filters is None:
-            filters = {}
-        instances_with_allowed = self._get_all_stored_model_instances(model, filters)
+    def _get_model_instances(self, model, filters):
+        
+        instances = self._get_stored_model_instances(model, filters)
         if self.save_consistency:
-            instances_with_allowed = self._check_fields_existence(model, instances_with_allowed, filters)
-        instances = self._get_instances_from_instances_with_allowed(instances_with_allowed)
+            instances = self._check_fields_existence(model, instances)
         return instances
     
-    def _get_all_stored_model_instances(self, model, filters=None):
+    def _get_stored_model_instances(self, model, filters):
         model_name = model.__name__
-        instances = self._get_instances_by_key(f'{self.prefix}:{model_name}:*')
-        instances_with_allowed = {}
-        for instance_id, instance_fields in instances.items():
-            for field_name, field_value in instance_fields.items():
-                allowed = self._filter_field_name(field_name, field_value, filters)
-                if instance_id not in instances_with_allowed.keys():
-                    instances_with_allowed[instance_id] = {}
-                instances_with_allowed[instance_id][field_name] = {
-                    'value': field_value,
-                    'allowed': allowed
-                }
-        return instances_with_allowed
+        if not filters:
+            instances = self._get_instances_data_by_ids(model)
+        else:
+            cleaned_filters = self._clean_filters(model, filters)
+            cleaned_filters_with_filtered_ids = self._get_cleaned_filters_with_filtered_ids(cleaned_filters)
+            starting_model_filtered_ids = self._get_starting_model_filtered_ids(cleaned_filters_with_filtered_ids)
+            instances = self._get_instances_data_by_ids(model, starting_model_filtered_ids)
+        return instances
+    
+    def _get_instances_data_by_ids(self, model, ids=None):
+        model_name = model.__name__
+        instances_data = {}
+        if ids is None:
+            raw_instances_data = self.fast_get_keys_values(f'{self.prefix}:{model_name}:*')
+            for raw_instance_key, raw_instance_value in raw_instances_data.items():
+                instance_id = int(raw_instance_key.split(':')[-2])
+                instnace_field_name = raw_instance_key.split(':')[-1]
+                if instance_id not in instances_data.keys():
+                    instances_data[instance_id] = {}
+                instances_data[instance_id][instnace_field_name] = raw_instance_value
+        else:
+            for instance_id in ids:
+                raw_fields_data = self.fast_get_keys_values(f'{self.prefix}:{model_name}:{instance_id}:*')
+                for field_key, field_value in raw_fields_data.items():
+                    if instance_id not in instances_data.keys():
+                        instances_data[instance_id] = {}
+                    instances_data[instance_id][field_key.split(':')[-1]] = field_value
+        instances_data = {
+            instance_id: {
+                field_name: self._deserialize_instance_field(model, field_name, field_value)
+                for field_name, field_value in raw_instance_fields.items()
+            }
+            for instance_id, raw_instance_fields in instances_data.copy().items()
+        }
+        return instances_data
     
     def _get_instances_by_key(self, key):
         raw_instances = self.fast_get_keys_values(key)
@@ -521,56 +537,33 @@ class RedisRoot:
             instance_id = int(instance_id)
             fields_dict = json.loads(fields_json)
             for field_name, raw_value in fields_dict.items():
-                value = self.deserialize_value(raw_value, model_name, field_name)
+                value = self._deserialize_instance_field(raw_value, model_name, field_name)
                 if instance_id not in instances.keys():
                     instances[instance_id] = {}
                 instances[instance_id][field_name] = value
         return instances
     
-    def _check_fields_existence(self, model, instances_with_allowed, filters):
+    def _check_fields_existence(self, model, instances):
         checked_instances = {}
         fields = model.get_class_fields()
         if 'id' not in fields.keys():
             fields['id'] = RedisString(null=True)
-        for instance_id, instance_fields in instances_with_allowed.items():
+        for instance_id, instance_fields in instances.items():
             checked_instances[instance_id] = {}
             for field_name, field in fields.items():
                 if field_name in instance_fields.keys():
                     checked_instances[instance_id][field_name] = instance_fields[field_name]
                 else:
-                    field.value = None
-                    cleaned_value = field.clean()
-                    allowed = self._filter_field_name(field_name, cleaned_value, filters)
-                    checked_instances[instance_id][field_name] = {
-                        'value': cleaned_value,
-                        'allowed': allowed
-                    }
+                    checked_instances[instance_id][field_name] = None
         return checked_instances
     
-    def _get_instances_from_instances_with_allowed(self, instances_with_allowed):
-        instances = {}
-        for instance_id, instance_fields in instances_with_allowed.items():
-            instance_allowed = all([
-                instance_field_data['allowed']
-                for instance_field_data in instance_fields.values()
-            ])
-            if instance_allowed and instance_id not in instances.keys():
-                instances[instance_id] = {
-                    instance_field_name: instance_field_data['value']
-                    for instance_field_name, instance_field_data in instance_fields.items()
-                }
-        return instances
-    
     ### DESERIALIZE ###
     
-    def deserialize_value(self, raw_value, model_name, field_name):
+    def _deserialize_instance_field(self, model, field_name, raw_value):
         value = raw_value
-        saved_model = self._get_registered_model_by_name(model_name)
-        if type(saved_model) != str:
-            if issubclass(saved_model, RedisModel):
-                saved_field_instance = self._get_field_instance_by_name(field_name, saved_model)
-                if issubclass(saved_field_instance.__class__, RedisField):
-                    value = self._deserialize_value_by_field_instance(raw_value, saved_field_instance)
+        saved_field_instance = self._get_field_instance_by_name(model, field_name)
+        if issubclass(saved_field_instance.__class__, RedisField):
+            value = self._deserialize_value_by_field_instance(saved_field_instance, raw_value)
         return value
     
     def _get_registered_model_by_name(self, model_name):
@@ -589,8 +582,7 @@ class RedisRoot:
                 raise Exception(f'{model_name} not found in registered models')
         return model
     
-    def _get_field_instance_by_name(self, field_name, model):
-        field_instance = None
+    def _get_field_instance_by_name(self, model, field_name):
         try:
             if field_name == 'id':
                 field_instance = getattr(model, field_name)
@@ -605,10 +597,10 @@ class RedisRoot:
                 raise Exception(f'{model.__name__} has no field {field_name}')
         return field_instance
     
-    def _deserialize_value_by_field_instance(self, raw_value, field_instance):
-        value = field_instance.deserialize_value(raw_value, self)
+    def _deserialize_value_by_field_instance(self, field_instance, raw_value):
+        
         try:
-            pass
+            value = field_instance.deserialize_value(raw_value, self)
         except BaseException as ex:
             if self.ignore_deserialization_errors:
                 print(
@@ -620,16 +612,29 @@ class RedisRoot:
     
     ### FILTER ###
     
-    def _filter_field_name(self, field_name, value, raw_filters):
-        allowed_list = [True]
-        for filter_param in raw_filters.keys():
-            filter_by = raw_filters[filter_param]
-            fields_to_filter, filter_type = self._split_filtering(filter_param)
-            if field_name == fields_to_filter[0]:
-                fields_to_filter = fields_to_filter[1:]
-                allowed_list.append(self._filter(value, fields_to_filter, filter_type, filter_by))
-        allowed = all(allowed_list)
-        return allowed
+    def _clean_filters(self, starting_model, filters):
+        cleaned_filters = {}
+        for filter_param, filter_value in filters.items():
+            field_to_filter_names, filter_type = self._split_filtering(filter_param)
+            filtering_models = [starting_model]
+            filtering_field_names = []
+            for field_to_filter_name in field_to_filter_names:
+                filtering_model_fields = filtering_models[-1].get_class_fields()
+                field = filtering_model_fields[field_to_filter_name]
+                if field.__class__ in [RedisForeignKey, RedisManyToMany]:
+                    filtering_models.append(field.model)
+                    filtering_field_names.append(field_to_filter_name)
+                else:
+                    key = json.dumps({
+                        'field_names': filtering_field_names,
+                        'model_names': [model.__name__ for model in filtering_models]
+                    })
+                    if key not in cleaned_filters.keys():
+                        cleaned_filters[key] = {}
+                    if field_to_filter_name not in cleaned_filters[key].keys():
+                        cleaned_filters[key][field_to_filter_name] = {}
+                    cleaned_filters[key][field_to_filter_name][filter_type] = filter_value
+        return cleaned_filters
     
     def _split_filtering(self, filter_param):
         filter_field_name, filter_type = filter_param, 'exact'
@@ -645,27 +650,35 @@ class RedisRoot:
             fields_to_filter = [filter_field_name]
         return fields_to_filter, filter_type
     
-    def _filter(self, value, fields_to_filter, filter_type, filter_by):
-        for field_to_filter in fields_to_filter:
-            if value in ['null', None]:
-                value = None
-            else:
-                try:
-                    value = value[field_to_filter]
-                except BaseException as ex:
-                    print(f'Exception: {ex}\n'
-                          f'Info: {field_to_filter}, {value}\n'
-                          f'Maybe: deep filtering is not included on this model')
-                    value = None
-        if isinstance(value, datetime.datetime) and isinstance(filter_by, datetime.datetime):
-            value = value.replace(tzinfo=pytz.UTC)
-            filter_by = filter_by.replace(tzinfo=pytz.UTC)
-        try:
-            allowed = self._filter_value(value, filter_type, filter_by)
-        except:
-            allowed = False
-        return allowed
-    
+    def _get_cleaned_filters_with_filtered_ids(self, cleaned_filters):
+        cleaned_filters_with_filtered_ids = {}
+        for relations_data, filter_data in cleaned_filters.items():
+            filtered_ids = []
+            filtering_model_name = json.loads(relations_data)['model_names'][-1]
+            for field_name, filters in filter_data.items():
+                stored_data = self.fast_get_keys_values(
+                    f'{self.prefix}:{filtering_model_name}:*:{field_name}'
+                )
+                model = self._get_registered_model_by_name(filtering_model_name)
+                field_filtered_ids = []
+                for instance_key, instance_value in stored_data.items():
+                    value = self._deserialize_instance_field(model, field_name, instance_value)
+                    allowed = [
+                        self._filter_value(value, filter_type, filter_by)
+                        for filter_type, filter_by in filters.items()
+                    ]
+                    if all(allowed):
+                        field_filtered_ids.append(int(instance_key.split(':')[-2]))
+                filtered_ids.append(field_filtered_ids)
+            if filtered_ids:
+                filtered_ids = list(reduce(
+                    lambda a, b: set(a & b),
+                    filtered_ids
+                ))
+            cleaned_filters_with_filtered_ids[relations_data] = filtered_ids
+        
+        return cleaned_filters_with_filtered_ids
+
     def _filter_value(self, value, filter_type, filter_by):
         allowed = True
         if filter_type == 'exact':
@@ -715,86 +728,166 @@ class RedisRoot:
                 allowed = False
         return allowed
     
+    def _get_starting_model_filtered_ids(self, cleaned_filters_with_filtered_ids):
+        starting_filtered_ids = []
+        for relations_data, filtered_ids in cleaned_filters_with_filtered_ids.items():
+            allowed_ids = filtered_ids.copy()
+            real_relations_data_copy = json.loads(relations_data)
+            while real_relations_data_copy['field_names']:
+                field_name = real_relations_data_copy['field_names'].pop(-1)
+                model_name = real_relations_data_copy['model_name'].pop(-1)
+                all_stored_model_fields = self.fast_get_keys_values(f'{self.prefix}:{model_name}:*:{field_name}')
+                allowed_ids = [
+                    int(instance_key.split(':')[-2])
+                    for instance_key, instance_value in all_stored_model_fields.items()
+                    if int(instance_value) in allowed_ids
+                ]
+            starting_filtered_ids.append(allowed_ids)
+        if starting_filtered_ids:
+            starting_filtered_ids = list(reduce(
+                lambda set_a, set_b: set((set_a & set_b)),
+                starting_filtered_ids.copy()
+            ))
+        return starting_filtered_ids
+        
     ### UPDATE ###
     
-    def update(self, model, instances=None, return_dict=False, renew_ttl=False, new_ttl=None, **fields_to_update):
-        model_name = model.__name__
-        instance_keys_to_update = self._get_instance_keys_to_update(instances, model_name)
-        updated_instances = {}
-        for instance_key in instance_keys_to_update:
-            prefix, model_name, instance_id = instance_key.split(':')
-            instance_id = int(instance_id)
-            fields_to_write = self._update_serialize_fields(instance_key, model, fields_to_update)
-            self._update_confirm(model, instance_key, renew_ttl, new_ttl, fields_to_write)
-            updated_instances[instance_id] = fields_to_write
+    def update(self, model, instances=None, return_dict=False, **fields_to_update):
+        updated_instances, data_to_update = self._collect_update(model, instances, fields_to_update)
+        self._confirm_update(data_to_update)
         result = self._return_with_format(updated_instances, return_dict)
         return result
     
-    def update_nb(self, model, instances=None, return_dict=False, renew_ttl=False, new_ttl=None, **fields_to_update):
-        model_name = model.__name__
-        instance_keys_to_update = self._get_instance_keys_to_update(instances, model_name)
-        updated_instances = {}
-        loop = asyncio.get_event_loop()
-        for instance_key in instance_keys_to_update:
-            prefix, model_name, instance_id = instance_key.split(':')
-            instance_id = int(instance_id)
-            fields_to_write = self._update_serialize_fields(instance_key, model, fields_to_update)
-            loop.create_task(
-                self._update_confirm_async(model, instance_key, renew_ttl, new_ttl, fields_to_write)
-            )
-            updated_instances[instance_id] = fields_to_write
+    def update_nb(self, model, instances=None, return_dict=False, **fields_to_update):
+        updated_instances, data_to_update = self._collect_update(model, instances, fields_to_update)
+        asyncio.get_event_loop().create_task(
+            self._confirm_update_async(data_to_update)
+        )
         result = self._return_with_format(updated_instances, return_dict)
         return result
     
-    def _get_instance_keys_to_update(self, instances, model_name):
-        keys_to_update = []
-        if self.use_keys:
-            if instances is None:
-                keys_to_update += list(self.redis_instance.keys(f'{self.prefix}:{model_name}:*'))
-            else:
-                ids_to_update = get_ids_from_untyped_data(instances)
-                for instance_id in ids_to_update:
-                    keys_to_update += list(self.redis_instance.keys(f'{self.prefix}:{model_name}:{instance_id}'))
+    def _collect_update(self, model, instances, fields_to_update):
+        model_name = model.__name__
+        if instances is not None:
+            ids_to_update = get_ids_from_untyped_data(instances)
         else:
-            if instances is None:
-                keys_to_update += list(self.redis_instance.scan_iter(f'{self.prefix}:{model_name}:*'))
-            else:
-                ids_to_update = get_ids_from_untyped_data(instances)
-                for instance_id in ids_to_update:
-                    keys_to_update += list(self.redis_instance.scan_iter(f'{self.prefix}:{model_name}:{instance_id}'))
-        return keys_to_update
+            ids_to_update = None
+        if ids_to_update is not None:
+            updated_instances = self.get(model, return_dict=True, id__in=ids_to_update)
+        else:
+            updated_instances = self.get(model, return_dict=True)
+        
+        collected_data_to_update = {}
+        for field_to_update_name, field_to_update_value in fields_to_update.items():
+            saved_field_instance = self._get_field_instance_by_name(model, field_to_update_name)
+            saved_field_instance.value = field_to_update_value
+            saved_field_instance.clean()
+            cleaned_field_to_update_value = saved_field_instance.value
+            updated_instances = {
+                updated_instance_id: {**updated_instance_data, field_to_update_name: field_to_update_value}
+                for updated_instance_id, updated_instance_data in updated_instances.items()
+            }
+            keys_to_update = self.collect_keys(model_name, ids_to_update, field_to_update_name)
+            update_mapping = {
+                key: cleaned_field_to_update_value
+                for key in keys_to_update
+            }
+            collected_data_to_update = {
+                **collected_data_to_update,
+                **update_mapping
+            }
+        return updated_instances, collected_data_to_update
     
-    def _update_serialize_fields(self, instance_key, model, fields_to_update):
-        instance_data_json = self.redis_instance.get(instance_key)
-        instance_data = json.loads(instance_data_json)
-        serialized_data = {}
-        for field_name, field_data in instance_data.items():
-            saved_field_instance = self._get_field_instance_by_name(field_name, model)
-            if field_name in fields_to_update.keys():
-                saved_field_instance.value = fields_to_update[field_name]
-                cleaned_value = saved_field_instance.clean()
-            else:
-                cleaned_value = field_data
-            serialized_data[field_name] = cleaned_value
-        return serialized_data
+    def _confirm_update(self, data_to_update):
+        if data_to_update.keys():
+            self.redis_instance.mset(data_to_update)
     
-    def _update_confirm(self, model, instance_key, renew_ttl, new_ttl, fields_to_write):
-        ttl = None
-        if renew_ttl:
-            ttl = model.get_instance_ttl()
-        elif new_ttl:
-            ttl = new_ttl
-        fields_to_write_json = json.dumps(fields_to_write)
-        self.redis_instance.set(instance_key, fields_to_write_json, ex=ttl)
+    async def _confirm_update_async(self, data_to_update):
+        self._confirm_update(data_to_update)
+            
+    #
+    #
+    # def _get_instance_keys_to_update(self, instances, model_name):
+    #     keys_to_update = []
+    #
+    #     if self.use_keys:
+    #         if instances is None:
+    #             keys_to_update += list(self.redis_instance.keys(f'{self.prefix}:{model_name}:*'))
+    #         else:
+    #             ids_to_update = get_ids_from_untyped_data(instances)
+    #             for instance_id in ids_to_update:
+    #                 keys_to_update += list(self.redis_instance.keys(f'{self.prefix}:{model_name}:{instance_id}:*'))
+    #     else:
+    #         if instances is None:
+    #             keys_to_update += list(self.redis_instance.scan_iter(f'{self.prefix}:{model_name}:*'))
+    #         else:
+    #             ids_to_update = get_ids_from_untyped_data(instances)
+    #             for instance_id in ids_to_update:
+    #                 keys_to_update += list(self.redis_instance.scan_iter(f'{self.prefix}:{model_name}:{instance_id}:*'))
+    #     return keys_to_update
     
-    async def _update_confirm_async(self, model, instance_key, renew_ttl, new_ttl, fields_to_write):
-        ttl = None
-        if renew_ttl:
-            ttl = model.get_instance_ttl()
-        elif new_ttl:
-            ttl = new_ttl
-        fields_to_write_json = json.dumps(fields_to_write)
-        self.redis_instance.set(instance_key, fields_to_write_json, ex=ttl)
+    # def update(self, model, instances=None, return_dict=False, renew_ttl=False, new_ttl=None, **fields_to_update):
+    #     model_name = model.__name__
+    #     instance_keys_to_update = self._get_instance_keys_to_update(instances, model_name)
+    #     updated_instances = {}
+    #     for instance_key in instance_keys_to_update:
+    #         prefix, model_name, instance_id = instance_key.split(':')
+    #         instance_id = int(instance_id)
+    #         fields_to_write = self._update_serialize_fields(instance_key, model, fields_to_update)
+    #         self._update_confirm(model, instance_key, renew_ttl, new_ttl, fields_to_write)
+    #         updated_instances[instance_id] = fields_to_write
+    #     result = self._return_with_format(updated_instances, return_dict)
+    #     return result
+    #
+    # def update_nb(self, model, instances=None, return_dict=False, renew_ttl=False, new_ttl=None, **fields_to_update):
+    #     model_name = model.__name__
+    #     instance_keys_to_update = self._get_instance_keys_to_update(instances, model_name)
+    #     updated_instances = {}
+    #     loop = asyncio.get_event_loop()
+    #     for instance_key in instance_keys_to_update:
+    #         prefix, model_name, instance_id = instance_key.split(':')
+    #         instance_id = int(instance_id)
+    #         fields_to_write = self._update_serialize_fields(instance_key, model, fields_to_update)
+    #         loop.create_task(
+    #             self._update_confirm_async(model, instance_key, renew_ttl, new_ttl, fields_to_write)
+    #         )
+    #         updated_instances[instance_id] = fields_to_write
+    #     result = self._return_with_format(updated_instances, return_dict)
+    #     return result
+    #
+
+    #
+    # def _update_serialize_fields(self, instance_key, model, fields_to_update):
+    #     instance_data_json = self.redis_instance.get(instance_key)
+    #     instance_data = json.loads(instance_data_json)
+    #     serialized_data = {}
+    #     for field_name, field_data in instance_data.items():
+    #         saved_field_instance = self._get_field_instance_by_name(field_name, model)
+    #         if field_name in fields_to_update.keys():
+    #             saved_field_instance.value = fields_to_update[field_name]
+    #             cleaned_value = saved_field_instance.clean()
+    #         else:
+    #             cleaned_value = field_data
+    #         serialized_data[field_name] = cleaned_value
+    #     return serialized_data
+    #
+    # def _update_confirm(self, model, instance_key, renew_ttl, new_ttl, fields_to_write):
+    #     ttl = None
+    #     if renew_ttl:
+    #         ttl = model.get_instance_ttl()
+    #     elif new_ttl:
+    #         ttl = new_ttl
+    #     fields_to_write_json = json.dumps(fields_to_write)
+    #     self.redis_instance.set(instance_key, fields_to_write_json, ex=ttl)
+    #
+    # async def _update_confirm_async(self, model, instance_key, renew_ttl, new_ttl, fields_to_write):
+    #     ttl = None
+    #     if renew_ttl:
+    #         ttl = model.get_instance_ttl()
+    #     elif new_ttl:
+    #         ttl = new_ttl
+    #     fields_to_write_json = json.dumps(fields_to_write)
+    #     self.redis_instance.set(instance_key, fields_to_write_json, ex=ttl)
     
     ### DELETE ###
     
@@ -809,14 +902,12 @@ class RedisRoot:
         )
     
     def _confirm_delete(self, model_name, instances):
-        if instances is None:
-            for key in self.redis_instance.keys(f'{self.prefix}:{model_name}:*'):
-                self._delete_by_key(key)
-        else:
+        ids_to_delete = None
+        if instances is not None:
             ids_to_delete = get_ids_from_untyped_data(instances)
-            for instance_id in ids_to_delete:
-                for key in self.redis_instance.keys(f'{self.prefix}:{model_name}:{instance_id}'):
-                    self._delete_by_key(key)
+        keys_to_delete = self.collect_keys(model_name, ids_to_delete)
+        if keys_to_delete:
+            self.redis_instance.delete(*keys_to_delete)
     
     async def _confirm_delete_async(self, model_name, instances):
         self._confirm_delete(model_name, instances)
@@ -867,6 +958,24 @@ class RedisRoot:
         values = self.redis_instance.mget(keys)
         results = dict(zip(keys, values))
         return results
+    
+    def collect_keys(self, model_name, ids=None, field_name=None):
+        collected_keys = []
+        field_name_query_string = '*' if field_name is None else field_name
+        if ids is None:
+            query_string = f'{self.prefix}:{model_name}:*:{field_name_query_string}'
+            if self.use_keys:
+                collected_keys = list(self.redis_instance.keys(query_string))
+            else:
+                collected_keys = list(self.redis_instance.scan_iter(query_string))
+        else:
+            for id in ids:
+                query_string = f'{self.prefix}:{model_name}:{id}:{field_name_query_string}'
+                if self.use_keys:
+                    collected_keys += list(self.redis_instance.keys(query_string))
+                else:
+                    collected_keys += list(self.redis_instance.scan_iter(query_string))
+        return collected_keys
 
 
 ### REDIS MODEL ###
@@ -901,7 +1010,6 @@ class RedisModel:
         fields = {}
         for field_name, field in class_fields.items():
             fields[field_name] = self._get_initial_model_field(field_name)
-        self._set_meta(self.__class__.get_class_meta())
         self.__model_data__['fields'] = fields
     
     @classmethod
@@ -913,31 +1021,7 @@ class RedisModel:
             if isinstance(field_value, RedisField):
                 fields[field_name] = field_value
         return fields
-    
-    @classmethod
-    def get_class_meta(cls):
-        field_names = dir(cls)
-        meta_fields = {}
-        if 'Meta' in field_names:
-            meta = cls.Meta
-            meta_field_names = dir(meta)
-            meta_fields = {meta_param_name: getattr(meta, meta_param_name) for meta_param_name in meta_field_names}
-        return meta_fields
-    
-    def _set_meta(self, meta_fields):
-        allowed_meta_fields_with_check_functions = {
-            'ttl': self._check_meta_ttl,
-        }
-        for field_name, field_value in meta_fields.items():
-            if field_name in allowed_meta_fields_with_check_functions.keys():
-                cleaned_value = allowed_meta_fields_with_check_functions[field_name](field_value)
-                if cleaned_value is not None:
-                    self.__model_data__['meta'][field_name] = cleaned_value
-    
-    def _check_meta_ttl(self, ttl):
-        check_types(ttl, (int, float))
-        return ttl
-    
+        
     def _get_initial_model_field(self, field_name):
         name = self.get('name')
         if field_name in dir(self.__class__):
@@ -963,7 +1047,7 @@ class RedisModel:
     def save_nb(self):
         instance_key, fields_dict, deserialized_fields = self._serialize_data()
         asyncio.get_event_loop().create_task(
-            self._set_field_async(instance_key, fields_dict)
+            self._set_fields_async(instance_key, fields_dict)
         )
         return deserialized_fields
     
@@ -981,7 +1065,7 @@ class RedisModel:
             try:
                 cleaned_value = field.clean()
                 cleaned_fields[field_name] = cleaned_value
-                deserialized_value = redis_root.deserialize_value(cleaned_value, name, field_name)
+                deserialized_value = redis_root._deserialize_instance_field(self, field_name, cleaned_value)
                 deserialized_fields[field_name] = deserialized_value
             except BaseException as ex:
                 raise Exception(f'{ex} ({name} -> {field_name})')
@@ -992,25 +1076,19 @@ class RedisModel:
         self.id.value = redis_root.get_and_reserve_new_id(self.__class__)
     
     def _set_fields(self, instance_key, fields_dict):
-        model_ttl = self.get_instance_ttl()
         redis_root = self.get('redis_root')
         prefix, model_name, instance_id = instance_key.split(':')
         instance_id = int(instance_id)
-        redis_instance = redis_root.redis_instance
-        fields_json = json.dumps(fields_dict)
-        redis_instance.set(instance_key, fields_json, ex=model_ttl)
+        fields_dict = {
+            f'{instance_key}:{field_name}': field_value
+            for field_name, field_value in fields_dict.copy().items()
+        }
+        redis_root.redis_instance.mset(fields_dict)
         redis_root.remove_creating(self.__class__, instance_id)
     
-    async def _set_field_async(self, instance_key, fields_dict):
+    async def _set_fields_async(self, instance_key, fields_dict):
         self._set_fields(instance_key, fields_dict)
-    
-    def get_instance_ttl(self):
-        ttl = None
-        meta = self.get('meta')
-        if 'ttl' in meta.keys():
-            ttl = meta['ttl']
-        return ttl
-    
+        
     ### UTILS ###
     
     def set(self, force=False, **fields_with_values):
